# Replace SiteTemplate with your plugin name.
# The package name must exactly match the file name (.pm)
package GCPlugins::GCbooks::GCNKC;

###################################################
#
#  Copyright 2005-2009 Tian
#  Copyright 2007,2011 Petr Gajdůšek <gajdusek.petr@centrum.cz>
#
#  This file is part of GCstar.
#
#  GCstar is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  GCstar is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with GCstar; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###################################################

# BUG:  Single list fields: values are stored as comma delimited string as plugin hangs otherwise
#       because there is some problem with strings containing diacritics.
#       Storing values as comma delimited string works but has an issue
#       while values contain commas and therefore are split.
# BUG:  When searching for title results are limited to the first page as I do not know how to handle more pages.
# BUG:  Many parsed fields are not supported by the model, so they are stored in description field.
# BUG:  Czech translation of some fields in model is wrong
# BUG:  Link to the source page and maybe refresh will probably not work always due the session expire (?)
# TODO: Make enhanced model
# TODO: Parse MARC records instead of sandard HTML view

use strict;
#use warnings;
use utf8;

use GCPlugins::GCbooks::GCbooksCommon;

{

    package GCPlugins::GCbooks::GCPluginNKC;

    use base qw(GCPlugins::GCbooks::GCbooksPluginsBase);

    sub getSearchCharset
    {
        return 'UTF-8';
    }

    sub getSearchFieldsArray
    {
        return [ 'isbn', 'title' ];
    }

    sub getSearchUrl
    {
        my ($self, $word) = @_;

        if ($self->{searchField} eq 'isbn')
        {
            return "http://aleph.nkp.cz/F?func=find-b&find_code=ISN&request=" . $word;
        }
        elsif ($self->{searchField} eq 'title')
        {
            return "http://aleph.nkp.cz/F/?func=find-b&find_code=WTL&request=" . $word;
        }
    }

    sub getItemUrl
    {
        my ($self, $url) = @_;

        # does not work (why?)
        $url = "http://aleph.nkp.cz" . $url if ($url !~ /^http:/);
        return $url;
    }

    sub getName
    {
        return "Národní knihovna (CS)";
    }

    sub getAuthor
    {
        return 'Petr Gajdůšek';
    }

    sub getLang
    {
        return 'CS';
    }

    # Can be used to change URL if movie URL and the one used to
    # extract information are different.
    # Return the modified URL.
    sub changeUrl
    {
        my ($self, $url) = @_;

        return $url;
    }

    sub preProcess
    {
        my ($self, $html) = @_;

        $self->{parsingEnded}  = 0;
        $self->{insideTD}      = 0;
        $self->{columnCounter} = 0;

        if ($self->{parsingList})
        {
            $self->{itemIdx} = -1;

            $html =~ s|&nbsp;| |gi;
            $html =~ s|<script>.*?</script>||gi;
            $html =~ s|<span.*?>||gi;
            $html =~ s|</span>||gi;
            $html =~ s|<IMG .*?>||gi;
            $html =~ s|<BR( /)?>|\n|gi;

            print $html, "\n" if ($self->{debug});
            print "searchField: ", $self->{searchField}, "\n";
            print "Loaded URL: ",  $self->{loadedUrl},   "\n";
        }
        else
        {
            $self->{insideAuthors}             = 0;
            $self->{insideTitle}               = 0;
            $self->{insidePublisher}           = 0;
            $self->{insideGenre}               = 0;
            $self->{insideKeywords}            = 0;
            $self->{insidePhysicalDescription} = 0;
            $self->{insideISBN}                = 0;
            $self->{insideExtraISBN}           = 0;
            $self->{insideEdition}             = 0;
            $self->{insideSerie}               = 0;
            $self->{insideOtherTitle}          = 0;
            $self->{insideOriginalTitle}       = 0;
            $self->{insideConspectusGroup}     = 0;
            $self->{insideContent}             = 0;
            $self->{insideCorporation}         = 0;
            $self->{insideResume}              = 0;

            $self->{empty} = 0;

            $html =~ s|&nbsp;| |gi;
            $html =~ s|<script>.*?</script>||gi;
            $html =~ s|<span.*?>||gi;
            $html =~ s|</span>||gi;
            $html =~ s|<A .*?>||gi;
            $html =~ s|</A>||gi;
            $html =~ s|<IMG .*?>||gi;
            $html =~ s|<BR( /)?>|\n|gi;

            #print $html,"\n" if ($self->{debug});
        }

        return $html;
    }

    sub start
    {
        my ($self, $tagname, $attr, $attrseq, $origtext) = @_;
        $self->{inside}->{$tagname}++;

        if ($tagname eq "td")
        {
            $self->{insideTD} = 1 if ($attr->{class} eq "td1");
            $self->{columnCounter}++;
        }
        $self->{columnCounter} = -1 if ($tagname eq "tr");

        if ($self->{parsingList})
        {
            return if !$self->{insideTD};

            if ($tagname eq 'a' and $self->{columnCounter} eq 0)
            {
                $self->{itemIdx}++;
                $self->{itemsList}[ $self->{itemIdx} ]->{url} = $attr->{href};
            }

        }
        else
        {
        }
    }

    sub text
    {
        my ($self, $origtext) = @_;

        return if ($self->{parsingEnded});
        $origtext =~ s/^\s+|\s+$//g;

        if ($self->{parsingList})
        {
            if ($self->{inside}->{title} and $origtext =~ m/Úplné zobrazení záznamu/i)
            {
                $self->{parsingEnded}        = 1;
                $self->{itemIdx}             = 0;
                $self->{itemsList}[0]->{url} = $self->{loadedUrl};
            }

            return if length($origtext) == 0;
            return if !$self->{insideTD};

            if ($self->{columnCounter} eq 2)
            {
                $self->{itemsList}[ $self->{itemIdx} ]->{title} = $origtext;
            }
            elsif ($self->{columnCounter} eq 3)
            {
                $self->{itemsList}[ $self->{itemIdx} ]->{authors} = $origtext;
            }
            elsif ($self->{columnCounter} eq 4)
            {
                $self->{itemsList}[ $self->{itemIdx} ]->{publisher} = $origtext;
            }
            elsif ($self->{columnCounter} eq 5)
            {
                $self->{itemsList}[ $self->{itemIdx} ]->{publication} = $origtext;
            }

        }
        else
        {

            if ($self->{insideTD})
            {
                $self->{empty} = 0 if ($origtext ne "");
                $self->{empty}++ if ($origtext eq "");

                # End of rows describing single term (non-empty first column)
                if (!$self->{empty} and $self->{columnCounter} == 0)
                {
                    $self->{insideKeywords}         = 0;
                    $self->{insideNotices}          = 0;
                    $self->{insideGenre}            = 0;
                    $self->{insideChronologicEntry} = 0;
                    $self->{insideAuthors}          = 0;
                    $self->{insideContent}          = 0;
                    $self->{insideExtraISBN}        = 0;
                    $self->{insideConspectusGroup}  = 0;
                    $self->{insideCorporation}      = 0;
                    $self->{insideOtherTitle}       = 0;
                    $self->{insideResume}           = 0;
                }

                return if length($origtext) == 0;

                ### Setting fields

                ## Simple fields

                # Fields to store as arrays
                foreach (
                    qw/authors genre keywords extraISBN otherTitle
                    conspectusGroup content chronologicEntries corporation resume/
                  )
                {
                    push @{$self->{curInfo}->{$_}}, $origtext
                      if ($self->{'inside' . ucfirst $_});
                }
                # Fields to store as single string
                foreach (qw/edition originalTitle/)
                {
                    $self->{curInfo}->{$_} = $origtext
                      if ($self->{'inside' . ucfirst $_});
                }
                $self->{$_} = 0 foreach (qw/insideEdition insideOriginalTitle/);

                ## Fields requiring complex parsing

                # Title
                if ($self->{insideTitle})
                {
                    $self->{insideTitle} = 0;
                    $self->{curInfo}->{title} = $origtext;
                    my ($translator) = $origtext =~ m|\[.*?\bpřeložil.?\s+(.*?)\s*\]|i;
                    $self->{curInfo}->{translator} = $translator if ($translator);
                }

                # Publisher and the date of plublication
                if ($self->{insidePublisher})
                {
                    $self->{insidePublisher} = 0;
                    ($self->{curInfo}->{publisher}, $self->{curInfo}->{publication}) =
                      $origtext =~ m/^.*?: (.*?),\s*\[?([0-9]+)\]?/;
                }

                # Notices
                if ($self->{insideNotices})
                {
                    push @{$self->{curInfo}->{notices}}, $origtext
                      if ($origtext !~ m/^poznámk.*?\b$/i);
                }

                # Physical description
                if ($self->{insidePhysicalDescription})
                {
                    $self->{insidePhysicalDescription} = 0;
                    # pages
                    ($self->{curInfo}->{pages}) = $origtext =~ m/\b([0-9]+)\s+s\./;
                    # dimenstion
                    ($self->{curInfo}->{dimension}) =
                      $origtext =~ m/(\b[0-9\.,]+\s*[cm]m\b)/;
                    # other physical describtions
                    $origtext =~ s/\b[0-9, ]+\s+s\.//;                 # remove pages
                    $origtext =~ s/\b[0-9\.,]+\s*[cm]m\b//;            # remove dimension
                    $origtext =~ s/\s+/ /g;
                    $origtext =~ s/\s*;\s*;\s*/ ; /g;
                    $origtext =~ s/(^\s*[;:]*\s*)|(\s*[;:]*\s*$)//g;
                    $self->{curInfo}->{physicalDescription} = $origtext if ($origtext);
                }

                # ISBN, format and price
                if ($self->{insideISBN})
                {
                    # format
                    $origtext =~ s/^\*\s*//;
                    my ($format) = $origtext =~ m/\((.*?)\)/;
                    $format = $1
                      if (index($format, ':') >= 0 and $format =~ m/:\s*([^:]+)$/);
                    $format = '' if ($format =~ m/\bchyb\./i);
                    $self->{curInfo}->{format} = $format if ($format);
                    # ISBN
                    ($self->{curInfo}->{isbn}) = $origtext =~ m/^([0-9X\-]+)/i;
                    # price
                    ($self->{curInfo}->{price}) =
                      $origtext =~ m/\s+:\s+(Kčs?\s+[0-9\.,]+)/i;
                    # Extra ISBN info
                    my $re = quotemeta($self->{curInfo}->{isbn}) . '\s*';
                    $re .= '\(\s*' . quotemeta($format) . '\s*\)' if ($format);
                    $re .= '\s*:\s*' . quotemeta($self->{curInfo}->{price})
                      if ($self->{curInfo}->{price});
                    if ($origtext !~ m|^$re$|i)
                    {
                        $origtext =~ s/^\s*\*\s*//;
                        my @tmp = split(/\s*\*\s*/, $origtext);
                        push @{$self->{curInfo}->{extraISBN}}, @tmp;
                        $self->{insideExtraISBN} = 1;
                    }

                    $self->{insideISBN} = 0;
                }

                # Serie ("Edice" in Czech)
                if ($self->{insideSerie})
                {
                    my @items = split(/\s*;\s*/, $origtext);
                    $self->{curInfo}->{serie} = $items[0];
                    # default / unknown rank (changed GCStar default 0 as it is misleading if serie is known
                    $self->{curInfo}->{rank} = 0;
                    $self->{curInfo}->{rank} = $1
                      if ($origtext =~ m/;[^;]*?([0-9]+)[^0-9]*$/);
                    $self->{insideSerie} = 0;
                }

                # Main author
                $self->{insideAuthors} = 1 if ($origtext =~ m/^Hlavní záhlaví$/i);
                # Other authors (translaters etc.)
                $self->{insideAuthors} = 1 if ($origtext =~ m/^Další původce$/i);
                # Full title with author, translater and other info
                $self->{insideTitle} = 1 if ($origtext =~ m/^Název$/i);
                # Publisher and the date of plublication
                $self->{insidePublisher} = 1 if ($origtext =~ m/^Nakl. údaje$/i);
                # Notices
                $self->{insideNotices} = 1 if ($origtext =~ m/^Poznámka$/i);
                # Genres
                $self->{insideGenre} = 1 if ($origtext =~ m/^Forma, žánr$/i);
                # Keywords
                $self->{insideKeywords} = 1 if ($origtext =~ m/^Předmět. heslo$/i);
                # Physical description (pages, dimension, other info)
                $self->{insidePhysicalDescription} = 1
                  if ($origtext =~ m/^Popis \(rozsah\)$/i);
                # ISBN, format and price
                $self->{insideISBN} = 1 if ($origtext =~ m/^ISBN$/i);
                # Serie ("Edice" in Czech)
                $self->{insideSerie} = 1 if ($origtext =~ m/^Edice$/i); # "Edice" in Czech
                       # Edition (release/emission, "Vydání in Czech
                $self->{insideEdition} = 1
                  if ($origtext =~ m/^Vydání$/i);    # "Vydání" in Czech
                                                       # Title variants
                $self->{insideOtherTitle} = 1 if ($origtext =~ m/^Varianta názvu$/i);
                # Original title in case of translation
                $self->{insideOriginalTitle} = 1
                  if ($origtext =~ m/^Název originálu$/i);
                # Conspectus group
                $self->{insideConspectusGroup} = 1
                  if ($origtext =~ m/^Skupina Konspektu$/i);
                # Content
                $self->{insideContent} = 1 if ($origtext =~ m/^Obsahuje/i);
                # Chronologic entries
                $self->{insideChronologicEntry} = 1
                  if ($origtext =~ m/^Chronolog. údaj$/i);
                # Corporations except publisher (including editor if different from the publisher)
                $self->{insideCorporation} = 1 if ($origtext =~ m/^Další korporace$/i);
                # Resumé
                $self->{insideResume} = 1 if ($origtext =~ m/^Resumé/i);
            }
        }
    }

    sub end
    {
        my ($self, $tagname) = @_;
        $self->{inside}->{$tagname}--;

        $self->{insideTD}      = 0  if ($tagname eq "td");
        $self->{columnCounter} = -1 if ($tagname eq "tr");

        if ($self->{parsingList})
        {
        }
        else
        {
            # finished parsing
            if ($tagname eq "body")
            {
                ## split fields that can contain more items/lines, delimiter may be * or newline
                $self->split_fields;

                ## setting of fields depending on other fields content (i.e. finding editor in corporations list)
                foreach my $field ('corporation', 'otherTitles')
                {
                    if ($self->{curInfo}->{$field})
                    {
                        foreach my $index (0 .. $#{$self->{curInfo}->{$field}})
                        {
                            my $val = $self->{curInfo}->{$field}->[$index];
                          # Find editor ("Vydavatelství" in Czech) in parsed corporations
                            if ($field eq 'corporation')
                            {
                                if ($val =~ m/s*\(\s*vyda.*?\)/i)
                                {
                                    $val =~ s/s*\(\s*vyda.*?\)\s*//g;
                                    push @{$self->{curInfo}->{editor}}, $val;
                                    delete $self->{curInfo}->{$field}->[$index];
                                }
                            }
                            # remove garbage from other titles and those titles same as main title
                            elsif ($field eq 'otherTitles')
                            {
                                $val =~ s/\s*[\.,]\s*Česky$//i;
                                $self->{curInfo}->{$field}->[$index] = $val;
                                delete $self->{curInfo}->{$field}->[$index]
                                  if ($val eq $self->{curInfo}->{title});
                            }
                        }
                    }
                }

                ## one more time split items, trim them and remove empty ones
                $self->split_fields;

                ## append genre related not supported list fields to genres
                foreach my $field ('keywords', 'chronologicEntries')
                {
                    push @{$self->{curInfo}->{genre}}, @{$self->{curInfo}->{$field}}
                      if $self->{curInfo}->{$field};
                }

                ## append not supported fields to the description field
                my %desc = (
                    resume             => "Resumé",
                    content            => "Obsahuje",
                    editor             => "Vydavatelství",
                    notices            => "Poznámky",
                    originalTitle      => "Název originálu",
                    otherTitles        => "Varianty názvu",
                    keywords           => "Předmět. hesla",
                    chronologicEntries => "Chronologické údaje",
                    conspectusGroup    => "Skupina konspektu",
                    corporation        => "Další korporace",
                    extraISBN          => "ISBN (další informace)",
                    physicalDescription =>
                      "Další fyzický popis (kromě počtu stran a rozměrů)",
                    dimension => "Rozměr",
                    price     => "Cena"

                );
                foreach my $field (keys %desc)
                {
                    if ($self->{curInfo}->{$field})
                    {
                        if (ref($self->{curInfo}->{$field}) eq 'ARRAY')
                        {
                            $self->{curInfo}->{description} .=
                              $desc{$field} . ":\n\t"
                              . join("\n\t", @{$self->{curInfo}->{$field}}) . "\n\n";
                        }
                        else
                        {
                            $self->{curInfo}->{description} .=
                                $desc{$field} . ":\n\t"
                              . $self->{curInfo}->{$field} . "\n\n";

                        }
                    }
                }

                ## Combine field parts into single string field
                $self->{curInfo}->{notices} = join("\n", @{$self->{curInfo}->{notices}})
                  if ($self->{curInfo}->{notices});
                $self->{curInfo}->{content} = join("\n", @{$self->{curInfo}->{content}})
                  if ($self->{curInfo}->{content});

                ## convert lists to comma delimited strings as plugin hangs if lists contain diacritic
                foreach my $field (keys %{$self->{curInfo}})
                {
                    if (ref($self->{curInfo}->{$field}) eq 'ARRAY')
                    {
                        map { s/,/ --/g } @{$self->{curInfo}->{$field}};
                        $self->{curInfo}->{$field} =
                          join(", ", @{$self->{curInfo}->{$field}});
                    }
                }

                ## Debug if enabled
                if ($self->{debug})
                {
                    use Data::Dumper;
                    print Dumper $self->{curInfo};
                }
            }
        }
    }

    ## split fields that can contain more items/lines (delimiter may be * or newline), trim fields and remove empty ones
    sub split_fields
    {
        my $self = shift;
        foreach my $field (keys %{$self->{curInfo}})
        {
            if (ref($self->{curInfo}->{$field}) eq 'ARRAY')
            {
                @{$self->{curInfo}->{$field}} =
                  map { split(/(\n|\*)/) } @{$self->{curInfo}->{$field}};
                map { s/(^\s*|\s*$)//g } @{$self->{curInfo}->{$field}};
                @{$self->{curInfo}->{$field}} =
                  grep (!/^\*?$/, @{$self->{curInfo}->{$field}});
                delete $self->{curInfo}->{$field}
                  if (scalar @{$self->{curInfo}->{$field}} == 0);
            }
        }

    }

    # Returns object reference.
    sub new
    {
        my $proto = shift;
        my $class = ref($proto) || $proto;
        my $self  = $class->SUPER::new();

        # Do your init stuff here

        bless($self, $class);

        $self->{hasField} = {
            title       => 1,
            authors     => 1,
            publication => 1,
            format      => 0,
            edition     => 0,
            serie       => 0,
            # not used by model yet
            publisher   => 1,
            isbn        => 0,
            rank        => 0,
            genre       => 0,
            describtion => 0,
            pages       => 0,
            web         => 0,
            translator  => 0,
        };

        $self->{curName} = undef;
        $self->{curUrl}  = undef;

        $self->{debug} = ($ENV{GCS_DEBUG_PLUGIN_PHASE} > 0);

        return $self;
    }

}

1;
